-------------------------------------------------------------------------------
--    File Name:  init.vhd
--      Version:  1.0
--         Date:  January 24, 2001
--  Description:  Initialization state machine for interface with Lara Networks CAM
--
--      Company:  Xilinx
--
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER.
--
--                Copyright (c) 2000 Xilinx, Inc.
--                All rights reserved
--
--   NOTE: for simulation purposes, comment generic state machine encodings
--         and uncomment constant state machine encodings.
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;

entity init is
  
  generic (
    ---------------------------------------------------------------------------
    -- valid values for generic values
    -- for datawidth = 00, for 68-bit
    --                 01, for 136-bit
    --                 10, for 272-bit
    -- for cascade_depth = any value within bounds, min is "00001"
    -- for  tlsz      = 00, for cascade_depth of 1
    --                  01, for cascade_depth of 2 to 8 devices
    --                  10, for cascade_depth of 9 to 31 devices
    ---------------------------------------------------------------------------
    datawidth  : std_logic_vector(1 downto 0) := "00";
    cascade_depth : std_logic_vector(4 downto 0) := "00001";
    tlsz : std_logic_vector(1 downto 0) := "00"   -- depth values used to build
                                                  -- data/address bus
    );
  
  port (
    clk       : in  std_logic;
    rst       : in  std_logic;
    start     : in  std_logic;          --start = 1 indicates that the initialize state machine can start execution
    last      : in  std_logic;          --last = 1 indicates last CAM to initialize
    id        : in  std_logic_vector(4 downto 0);
    cmmd      : out std_logic_vector(8 downto 0);  -- command
    dq        : out std_logic_vector (67 downto 0);  -- data/address bus
    cmmdv     : out std_logic;          -- command valid
    ready_i   : out std_logic;          -- ready_i indicates that the initialize state machine can start execution	
    i_tr_dq   : out std_logic;          -- tri_state enable signals
    i_tr_cmdv : out std_logic;
    i_tr_cmd  : out std_logic;
    id_enable : out std_logic);         -- id_enable = 1 indicates that the user can change the value of id

end init;




architecture init_arch of init is

  signal CurrState_initialize, NextState_initialize : std_logic_vector(2 downto 0);

constant idle : std_logic_vector(2 downto 0) := "000";
constant init2_cm  : std_logic_vector (2 downto 0) := "001"; 
constant init2_gmr : std_logic_vector (2 downto 0) := "010"; 
constant init3_cm  : std_logic_vector (2 downto 0) := "011"; 
constant init3_gmr : std_logic_vector (2 downto 0) := "100"; 
constant init_cm   : std_logic_vector (2 downto 0) := "101"; 
constant init_gmr  : std_logic_vector (2 downto 0) := "110";

begin  -- init_arch

-- create a process which creates outputs based on current state

  current_state_output: process (CurrState_initialize, id, last)
  begin
    case CurrState_initialize is
      when idle =>
        --tristate all outputs
        i_tr_dq   <= '1';
        i_tr_cmdv <= '1';
        i_tr_cmd  <= '1';
        ready_i   <= '1';

        id_enable <= '1';

        cmmd <=  (others => 'Z');
        cmmdv     <= 'Z';
        dq  <=  (others => 'Z');

      when init_cm =>
        i_tr_dq   <= '0';
        i_tr_cmdv <= '0';
        i_tr_cmd  <= '0';
        ready_i   <= '0';

        id_enable <= '0';

        --initialize command register
        --consult Lara Networks CAM datasheet
        --  for details on register meanings

        -- note that cmmd[2:0] indicates a single write,
        --  cmmd[8:3] are for parameters
        cmmd(1 downto 0) <= "01";
        cmmd(2)   <= '0';		--single write;
        cmmd(8 downto 3) <= "000000";
        cmmdv     <= '1';

        dq (67 downto 26) <= (others => '0');-- reserved command reg area
        dq (25 downto 21) <= id;-- give CAM ID
        dq (20 downto 19) <= "11";-- indicate internal register write
        dq (18 downto 6)  <= (others => '0');-- reserved command reg area
        dq (5 downto 0)   <= "111000";-- register address


      when init2_cm =>
        i_tr_dq   <= '0';
        i_tr_cmdv <= '0';
        i_tr_cmd  <= '0';
        ready_i   <= '0';

        id_enable <= '0';

        --initialize the command register
        cmmd(1 downto 0) <= "01";
        cmmd(2)   <= '0';		--single write;
        cmmd(8 downto 3) <= "000000";
        cmmdv     <= '0';

        dq(67 downto 66) <= "00";
        dq(65 downto 17) <= (others => '0');
        --table size configuration for each of the 4 quadrants
        dq(16 downto 9)  <= datawidth & datawidth & datawidth & datawidth;
        

	if (last = '1') then
          dq(8 downto 7) <= "11";	  --Last CAM
	else
          dq(8 downto 7) <= "00";
        end if;
        
        dq(6 downto 4) <= "000";  --0 latency for SSF, SSV, and ACK
        dq(3 downto 2) <= tlsz; -- table size
        dq(1) <= '1'; 	--device enable
        dq(0) <= '0'; 	--software reset if 1;


      when init3_cm =>
        i_tr_dq   <= '0';
        i_tr_cmdv <= '0';
        i_tr_cmd  <= '0';
        ready_i   <= '0';

        id_enable <= '0';

        -- allow one clock cycle to finish command reg write.
        cmmd(1 downto 0) <= "01";
        cmmd(2)   <= '0';		--single write;
        cmmd(8 downto 3) <= "000000";
        cmmdv     <= '0';

        dq(67 downto 0)  <= (others => 'Z');

      when init_gmr =>
      	i_tr_dq   <= '0';
        i_tr_cmdv <= '0';
        i_tr_cmd  <= '0';
        ready_i   <= '0';

        id_enable <= '0';

        --initialize Global Mask Reg
        -- note that cmmd[2:0] indicates a single write,
        --  cmmd[8:3] are for parameters
        -- allow one clock cycle to finish command reg write.
        cmmd(2 downto 0) <= "001";	--single write;
        cmmd(8 downto 3) <= "000000";
        cmmdv     <= '1';

        dq(67 downto 26) <= (others => '0');-- reserved
        dq(25 downto 21) <= id; --CAM ID
        dq(20 downto 19) <= "11";-- internal reg write
        dq(18 downto 6)  <= (others => '0');-- reserved
        dq(5 downto 0)   <= "100000";-- first global mask register

      when init2_gmr =>
      	i_tr_dq   <= '0';
        i_tr_cmdv <= '0';
        i_tr_cmd  <= '0';
        ready_i   <= '0';

        id_enable <= '1';

        --initialize Global Mask Reg
        cmmd(2 downto 0) <= "001";	--single write;
        cmmd(8 downto 3) <= "000000";
        cmmdv     <= '0';

        dq(67 downto 0)  <= (others => '1'); -- give global mask pattern

      when init3_gmr =>
        i_tr_dq   <= '0';
        i_tr_cmdv <= '0';
        i_tr_cmd  <= '0';
        ready_i   <= '1';

        id_enable <= '1';

        --initialize Global Mask Reg
        cmmd(2 downto 0) <= "001";	--single write;
        cmmd(8 downto 3) <= "000000";
        cmmdv     <= '0';

        dq(67 downto 0)  <= (others => 'Z');

      when others =>
        --tristate all outputs - emulate idle mode
        i_tr_dq   <= '1';
        i_tr_cmdv <= '1';
        i_tr_cmd  <= '1';
        ready_i   <= '1';

        id_enable <= '1';

        cmmd <=  (others => 'Z');
        cmmdv     <= 'Z';

        dq  <=  (others => 'Z');

    end case;

--    end if;      
  end process current_state_output;  


-- create a process that describes the next state logic

  next_state_logic: process (start, CurrState_initialize)
  begin

    case CurrState_initialize is

      when idle =>
        if(start = '1') then
          NextState_initialize <= init_cm;
        else
          NextState_initialize <= idle;
        end if;

      when init_cm =>

        NextState_initialize <= init2_cm;

      when init2_cm =>

        NextState_initialize <= init3_cm;

      when init3_cm =>

        NextState_initialize <= init_gmr;

      when init_gmr =>

        NextState_initialize <= init2_gmr;

      when init2_gmr =>

        NextState_initialize <= init3_gmr;

      when init3_gmr =>

	if (start = '1') then
          NextState_initialize <= init_cm;
	else
          NextState_initialize <=idle;
        end if;
      when others =>

        NextState_initialize <= idle;

    end case;

  end process next_state_logic;  


-- Current State Logic
  state_machine: process (clk, rst)
  begin  -- process state_machine
    if(rst = '1') then                     -- asynchronous reset 
      CurrState_initialize <= idle;
    elsif(clk'event and clk = '1') then    -- rising clock edge
      CurrState_initialize <= NextState_initialize;
    end if;
  end process state_machine;

end init_arch;
























