-------------------------------------------------------------------------------
--    File Name:  rw.vhd
--      Version:  1.0
--         Date:  January 24, 2001
--  Description:  Read/Write state machine for interface with Lara Networks CAM
--
--      Company:  Xilinx
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER.
--
--                Copyright (c) 2001 Xilinx, Inc.
--                All rights reserved
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


entity rw is

  port (
    clk          : in  std_logic;
    rst          : in  std_logic;
    eot_in       : in  std_logic;
    ack          : in  std_logic;
    user_command : in  std_logic_vector(5 downto 0);
    data_back    : in  std_logic_vector(67 downto 0);
    gmri         : in  std_logic_vector(2 downto 0);
    id           : in  std_logic_vector(4 downto 0);
    data         : in std_logic_vector(67 downto 0);  --output of register rw_in from top module
    cmmdv        : out std_logic;
    rw_tr_dq     : out std_logic;
    rw_tr_cmdv   : out std_logic;
    rw_tr_cmd    : out std_logic;
    ready_rw     : out std_logic;
    cmmd         : out std_logic_vector(8 downto 0);
    data_rw      : out std_logic_vector(67 downto 0);
    dq           : out std_logic_vector(67 downto 0));

end rw;

architecture rw_arch of rw is

-- define the state machine encodings
  constant    idle_a    : std_logic_vector(3 downto 0) := "0000";
  constant    read1     : std_logic_vector(3 downto 0) := "0001";
  constant    read2     : std_logic_vector(3 downto 0) := "0010";
  constant    read3     : std_logic_vector(3 downto 0) := "0011";
  constant    write1    : std_logic_vector(3 downto 0) := "0100";
  constant    write2    : std_logic_vector(3 downto 0) := "0101";
  constant    write3    : std_logic_vector(3 downto 0) := "0110";
  constant    write4    : std_logic_vector(3 downto 0) := "0111";
  constant    write5    : std_logic_vector(3 downto 0) := "1000";
  constant    write6    : std_logic_vector(3 downto 0) := "1001";
  constant    write_nop : std_logic_vector(3 downto 0) := "1010";



  
  signal burst_cnt_syn : std_logic_vector(8 downto 0);
  signal burst_cnt_com : std_logic_vector(8 downto 0);

  signal dmr : std_logic_vector(1 downto 0);

  signal burst        : std_logic;
  signal cmd          : std_logic_vector(1 downto 0);
  signal cmdv         : std_logic;
  signal burst_length : std_logic_vector(8 downto 0);  --data[23:15) = burst_length

-- the state machine
  signal CurrState_rw, NextState_rw: std_logic_vector(3 downto 0);

  signal data_rw_int : std_logic_vector(67 downto 0);


begin  -- rw_arch
-------------------------------------------------------------------------------
-- The user_command is broken down into the following:
-- user_command
--    (5)        (4 downto 3)      (2 downto 1)       (0)
--  burst         dmr                    cmd        cmdv
--
-- make some combinational assignments
-------------------------------------------------------------------------------  
  burst           <= user_command(5);
  dmr             <= user_command(4 downto 3);
  cmd(1 downto 0) <= user_command(2 downto 1);
  cmdv            <= user_command(0);

-- assign internal logic signals to outputs
  data_rw <= data_rw_int;


--NextState logic
  next_state_logic: process(CurrState_rw, cmd, cmdv, burst, eot_in, ack, burst_length, burst_cnt_syn)
  begin  -- process next_state_logic
    case CurrState_rw is
      when idle_a =>
        if (((cmd(1 downto 0) = "01") and (cmdv = '1')) or  -- a write
            ((cmd(1 downto 0) = "00") and (burst = '1') and (cmdv = '1'))) then -- a burst read
          -- Enter this state if you are doing a burst read
          -- (must program burst read reg first), or
          -- if you are doing a write (burst or not)
          NextState_rw <= write1;
        elsif ((cmd(1 downto 0) = "00") and (burst = '0') and (cmdv = '1')) then
          --Enter this state if you are doing a non-burst read.
          NextState_rw <= read1;
        else
          NextState_rw <= idle_a;
        end if;

      when read1 =>
        if(burst = '1') then
          NextState_rw <= read2;
        else
          NextState_rw <= read3;
        end if;

      when read2 =>
        if (eot_in = '1') then
          NextState_rw <= idle_a;
        else
          NextState_rw <= read2;
        end if;

      when read3 =>

        if (ack = '1') then
          NextState_rw <= idle_a;
        else
          NextState_rw <= read3;
        end if;

      when write1 =>
        NextState_rw <= write2;

      when write2 =>
        if ((cmd(1 downto 0) = "01") and (burst = '1')) then
          NextState_rw <= write_nop; --burst write
        elsif ((cmd(1 downto 0) = "00") and (burst= '1')) then
          NextState_rw <= write_nop; --burst read
        else
          NextState_rw <= idle_a; --finished single write
        end if;

      when write_nop =>
        if (cmd(1 downto 0) = "01") then
          NextState_rw <= write3;
        else
          NextState_rw <= read1;
        end if;

      when write3 => -- h6
        NextState_rw <= write4;
        
      when write4 => -- write4 = 4'b0111 = h7
        NextState_rw <= write5;

      when write5 => -- write5 = 4'b1000 = h8
        if (burst_cnt_syn = burst_length) then
          NextState_rw <= write6;
        else
          NextState_rw <= write5;
        end if;

      when write6 => -- write6 = 4'b1001 = h9
        NextState_rw <= idle_a;

      when others =>
        NextState_rw <= idle_a;
    end case;
    
  end process next_state_logic;


  current_state_output: process(cmd,burst, data, dmr, CurrState_rw, id, gmri)
  begin  -- process current_state_output
    
    case CurrState_rw is
      when idle_a =>
        --tri-state all outputs
        rw_tr_dq <=  '1';
        rw_tr_cmd <=  '1';
        rw_tr_cmdv <=  '1';
        ready_rw <=  '1';

        cmmd(8 downto 0) <= (others => 'Z');
        cmmdv <= 'Z';
        dq(67 downto 0) <= (others => 'Z');

      when read1 =>
        rw_tr_dq <= '0';
        rw_tr_cmd <= '0';
        rw_tr_cmdv <= '0';
        ready_rw <= '0';

        cmmd(1 downto 0) <= "00";
        cmmd(2) <= burst;
        cmmd(8 downto 3) <= "000000";
        cmmdv <=  '1';

        dq(67 downto 30) <= (others => '0');
        dq(29) <= '0';
        dq(28 downto 26) <= "000";
        dq(25 downto 21) <= id;
        
        -- 00 : data read, 01 : mask read, 10 : SRAM read, 11 : register read
        dq(20 downto 19) <= dmr; 

        dq(18 downto 15) <= (others => '0');

        --addr : for burst read, these bits are don't cares
        dq(14 downto 0) <= data(14 downto 0);

      when read2 =>
        --burst read only
        rw_tr_dq <=  '1';
        rw_tr_cmd <=  '1';
        rw_tr_cmdv <= '0';
        ready_rw <= '0';

        cmmd(1 downto 0) <= "00";
        cmmd(2) <= burst;
        cmmd(8 downto 3) <= "000000";

        cmmdv <= '0';

        dq(67 downto 30) <= (others => '0');
        dq(29) <= '0';
        dq(28 downto 26) <= "000";
        dq(25 downto 21) <= id;
        
        -- 00 : data read, 01 : mask read, 10 : SRAM read, 11 : register read
        dq(20 downto 19) <= dmr; 

        dq(18 downto 15) <= (others => '0');

        --addr : for burst read, these bits are don't cares
        dq(14 downto 0) <= data(14 downto 0);

        
      when read3 =>
        --single read or last cycle of burst read
        rw_tr_dq <=  '1';
        rw_tr_cmd <= '0';
        rw_tr_cmdv <= '0';
        ready_rw <=  '1';

        cmmd(1 downto 0) <= "00";
        cmmd(2) <= burst;
        cmmd(8 downto 3) <= "000000";
        cmmdv <= '0';

        dq(67 downto 30) <= (others => '0');
        dq(29) <= '0';
        dq(28 downto 26) <= "000";
        dq(25 downto 21) <= id;
        
        -- 00 : data read, 01 : mask read, 10 : SRAM read, 11 : register read
        dq(20 downto 19) <= dmr; 

        dq(18 downto 15) <= (others => '0');

        --addr : for burst read, these bits are don't cares
        dq(14 downto 0) <= data(14 downto 0);
        
      when write1 =>
        rw_tr_dq <= '0';
        rw_tr_cmd <= '0';
        rw_tr_cmdv <= '0';
        ready_rw <= '0';

        --write to burst read register
        if((cmd(1 downto 0) = "00") and (burst = '1')) then
          cmmdv <=  '1';
          cmmd(1 downto 0) <= "01";
          cmmd(2) <= '0';
          cmmd(8 downto 3) <= "000000";

          dq(5 downto 0) <= "111010"; --dec address 58 for burst read reg
          dq(18 downto 6) <= (others => '0');
          dq(20 downto 19) <= "11"; --for register write
          dq(25 downto 21) <= id;
          dq(28 downto 26) <= "000";
          dq(29) <= '0';
          dq(67 downto 30) <= (others => '0');
        elsif((cmd(1 downto 0) = "01") and (burst = '1')) then
          --write to burst write register
          cmmdv <=  '1';
          cmmd(2) <= '0';
          cmmd(1 downto 0) <= "01";
          cmmd(5 downto 3) <= gmri; --GMR index for write;
          cmmd(8 downto 6) <= "000";

          dq(5 downto 0) <= "111011"; --burst write reg address 59;
          dq(18 downto 6) <= (others => '0');
          dq(20 downto 19) <= "11";
          dq(25 downto 21) <= id;
          dq(28 downto 26) <= "000";
          dq(29) <= '0';
          dq(67 downto 30) <= (others => '0');
--      end if;

          --single write
        else -- if we're here, this can be nothing other than a single write
          cmmdv <=  '1';
          cmmd(2) <= '0';
          cmmd(1 downto 0) <= "01";
          cmmd(5 downto 3) <= gmri; --GMR index for write;
          cmmd(8 downto 6) <= "000";

          dq(14 downto 0) <= data(14 downto 0); --addr
          dq(18 downto 15) <= "0000";

          -- 00 : data write, 01 : mask write, 10 : SRAM write
          -- 11 : register write
          dq(20 downto 19) <= dmr;

          dq(25 downto 21) <= id;
          dq(28 downto 26) <= "000";
          dq(29) <= '0';
          dq(67 downto 30) <= (others => '0');
        end if;

      when write2 =>
        rw_tr_dq <= '0';
        rw_tr_cmd <= '0';
        rw_tr_cmdv <= '0';
        ready_rw <= '0';

        cmmdv <= '0';

        if ((cmd(1 downto 0) = "01") and (burst = '1')) then
          ready_rw <= '0';
          dq(14 downto 0) <= data(14 downto 0);
          dq(18 downto 15) <= (others => '0');
          dq(27 downto 19) <= data(23 downto 15);
          dq(67 downto 28) <= (others => '0');

          cmmd(2) <= '0';
          cmmd(1 downto 0) <= "01";
          cmmd(5 downto 3) <= gmri; --GMR index for write;
          cmmd(8 downto 6) <= "000";
        elsif ((cmd(1 downto 0) = "00") and (burst= '1')) then
          ready_rw <= '0';
          dq(14 downto 0) <= data(14 downto 0);
          dq(18 downto 15) <= (others => '0');
          dq(27 downto 19) <= data(23 downto 15);
          dq(67 downto 28) <= (others => '0');

          cmmd(1 downto 0) <= "01";
          cmmd(2) <= '0';
          cmmd(8 downto 3) <= "000000";
        else
          -- finished signal write
          ready_rw <=  '1';
          dq <= data;
          cmmd(2) <= '0';
          cmmd(1 downto 0) <= "01";
          cmmd(5 downto 3) <= gmri; --GMR index for write;
          cmmd(8 downto 6) <= "000";
        end if;

      when write_nop =>
        rw_tr_dq <=  '1';
        rw_tr_cmd <=  '1';
        rw_tr_cmdv <=  '1';
        ready_rw <= '0';
        cmmdv <= '0';
        if (cmd(1 downto 0) = "01") then
          dq(14 downto 0) <= data(14 downto 0);
          dq(18 downto 15) <= (others => '0');
          dq(27 downto 19) <= data(23 downto 15);
          dq(67 downto 28) <= (others => '0');
          cmmd(2) <= '0';
          cmmd(1 downto 0) <= "01";
          cmmd(5 downto 3) <= gmri; --GMR index for write;
          cmmd(8 downto 6) <= "000";
        else
          dq(14 downto 0) <= data(14 downto 0);
          dq(18 downto 15) <= (others => '0');
          dq(27 downto 19) <= data(23 downto 15);
          dq(67 downto 28) <= (others => '0');
          cmmd(1 downto 0) <= "01";
          cmmd(2) <= '0';
          cmmd(8 downto 3) <= "000000";
        end if;
      when write3 => -- h6
        rw_tr_dq <= '0';
        rw_tr_cmd <= '0';
        rw_tr_cmdv <= '0';
        ready_rw <= '0';

        --supply address
        cmmdv <=  '1';
        cmmd(1 downto 0) <= "01";
        cmmd(2) <= burst;
        cmmd(8 downto 3) <= "000000";

        dq(14 downto 0) <= data(14 downto 0); --address
        dq(18 downto 15) <= "0000";    -- reserved
        dq(20 downto 19) <= dmr;        -- data, mask, sram, or reg
        dq(25 downto 21) <= id;         -- cam ID
        dq(67 downto 26) <= (others => '0');

      when write4 => -- write4 = 4'b0111 = h7
        rw_tr_dq <= '0';
        rw_tr_cmd <= '0';
        rw_tr_cmdv <= '0';
        ready_rw <= '0';

        cmmdv <= '0';
        cmmd(1 downto 0) <= "01";
        cmmd(2) <= burst;
        cmmd(8 downto 3) <= "000000";

        --supply first data
        dq <= data;

      when write5 => -- write5 = 4'b1000 = h8
        rw_tr_dq <= '0';
        rw_tr_cmd <= '0';
        rw_tr_cmdv <= '0';
        ready_rw <= '0';

        cmmdv <= '0';
        cmmd(1 downto 0) <= "01";
        cmmd(2) <= burst;
        cmmd(8 downto 3) <= "000000";

        --suppy data
        dq <= data;

      when write6 => -- write6 = 4'b1001 = h9
        rw_tr_dq <=  '1';
        rw_tr_cmd <= '0';
        rw_tr_cmdv <= '0';
        ready_rw <=  '1';

        cmmdv <= '0';
        cmmd(1 downto 0) <= "01";
        cmmd(2) <= burst;
        cmmd(8 downto 3) <= "000000";

        dq <= data;
      when others =>
        --tri-state all outputs
        rw_tr_dq <=  '1';
        rw_tr_cmd <=  '1';
        rw_tr_cmdv <=  '1';
        ready_rw <=  '1';

        cmmd <= (others => 'Z');
        cmmdv <= 'Z';
        dq <= (others => 'Z');

    end case;

  end process current_state_output;


  registered_outputs: process(clk, rst)
  begin
    if(rst = '1') then
      data_rw_int <= (others => '0');
      burst_length <= (others => '0');
    elsif(clk'event and clk = '1') then
      if (((CurrState_rw = read2) or (CurrState_rw = read3)) and (ack = '1')) then
        data_rw_int <= data_back;
      end if;

      if(CurrState_rw = write2) then
        burst_length <= data(23 downto 15);
      end if;
    end if;
  end process registered_outputs;


  burst_count: process (clk, rst)
  begin  -- process burst_count
    if (rst = '1') then                     -- asynchronous reset
      burst_cnt_com <= "000000010";
      burst_cnt_syn <= "000000000";
    elsif clk'event and clk = '1' then    -- rising clock edge
      burst_cnt_syn <= burst_cnt_com;

      if (CurrState_rw = write4) then
        burst_cnt_com <= "000000010";
      elsif (CurrState_rw = write5) then
        burst_cnt_com <= burst_cnt_com + '1';
      end if;

    end if;
  end process burst_count;



-- Current State Logic
  state_machine: process (clk, rst)
  begin  -- process state_machine
    if (rst = '1') then                     -- asynchronous reset
      CurrState_rw  <= idle_a;
    elsif (clk'event and clk = '1') then    -- rising clock edge
      CurrState_rw  <= NextState_rw;
    end if;

  end process state_machine;




end rw_arch;

