-------------------------------------------------------------------------------
--
--    File Name:  search.vhd
--      Version:  1.0
--         Date:  January 24, 2001
--  Description:  Search state machine for interface with Lara Networks CAM
-- Dependencies:  
--
--      Company:  Xilinx
--
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER.
--
--                Copyright (c) 2000 Xilinx, Inc.
--                All rights reserved
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity search is

  generic (
    ---------------------------------------------------------------------------
    -- valid values for generic values
    -- for datawidth = 00, for 68-bit
    --                 01, for 136-bit
    --                 10, for 272-bit
    ---------------------------------------------------------------------------
    datawidth  : std_logic_vector(1 downto 0) := "00");
 
 
  port (
    clk2x               : in  std_logic;
    rst                 : in  std_logic;
    user_command        : in  std_logic_vector(5 downto 0);
    gmri                : in  std_logic_vector(2 downto 0);
    sadr                : in  std_logic_vector(2 downto 0);
    ssr_reg_index       : in  std_logic_vector(2 downto 0);
    comparand_reg_index : in  std_logic_vector(3 downto 0);
    data                : in  std_logic_vector(67 downto 0);
    cmmd                : out std_logic_vector(8 downto 0);
    dq                  : out std_logic_vector(67 downto 0);
    cmmdv               : out std_logic;
    s_tr_dq             : out std_logic;
    s_tr_cmdv           : out std_logic;
    s_tr_cmd            : out std_logic;
    ready_s             : out std_logic);

end search;



architecture search_arch of search is

constant    idle_s      : std_logic_vector(3 downto 0) := "0000";
constant    search136_a : std_logic_vector(3 downto 0) := "0001";
constant    search136_b : std_logic_vector(3 downto 0) := "0010";
constant    search272_a : std_logic_vector(3 downto 0) := "0011";
constant    search272_b : std_logic_vector(3 downto 0) := "0100";
constant    search272_c : std_logic_vector(3 downto 0) := "0101";
constant    search272_d : std_logic_vector(3 downto 0) := "0110";
constant    search68_a  : std_logic_vector(3 downto 0) := "0111";
constant    search68_b  : std_logic_vector(3 downto 0) := "1000"; 

  
signal cmd : std_logic_vector(1 downto 0);
signal cmdv : std_logic;

-- make the state machine
  signal CurrState_search, NextState_search : std_logic_vector(3 downto 0);

begin  -- search_arch

-- user_command
--    [5]        [4:3]      [2:1]       [0]
--  burst         dmr        cmd        cmdv

--assign combinatorial logic

  cmd <= user_command(2 downto 1);
  cmdv <= user_command(0);


  generate_68_bit_logic: if (datawidth = "00") generate 

  begin

    width68_nextstate: process (CurrState_search, cmd, cmdv)
    begin  -- process width68_nextstate
      case CurrState_search is
        when idle_s => 
                       if(cmd(1 downto 0) = "10" and cmdv = '1') then
                         NextState_search<=search68_a;
                       else
                         NextState_search<=idle_s;
                       end if;

        when search68_a =>
          NextState_search <= search68_b;

        when search68_b =>
          if (cmd(1 downto 0) = "10" and cmdv = '1') then
            NextState_search <= search68_a;
          else
            NextState_search <= idle_s;
          end if;
          
        when others =>                           NextState_search<=idle_s;

      end case;
    end process width68_nextstate;
    width68_logic: process(cmd, cmdv, comparand_reg_index, ssr_reg_index, CurrState_search, data, gmri, sadr)

    begin  -- process width68_logic
      case CurrState_search is
	when idle_s =>
          --tristate outputs

          s_tr_dq   <= '1'; 
          s_tr_cmdv <= '1'; 
          s_tr_cmd  <= '1';
          ready_s   <= '1';

          
          cmmd <= (others => 'Z');
          cmmdv     <= 'Z';
          dq  <= (others => 'Z');
          
          
	when search68_a =>
          s_tr_dq   <= '0'; 
          s_tr_cmdv <= '0'; 
          s_tr_cmd  <= '0';	
          ready_s   <= '0';
          
          cmmdv     <= '1';
          
          cmmd(8 downto 6) <= sadr; --SRAM addr for SRAM support
          cmmd(5 downto 3) <= gmri; 
          cmmd(2)	  <= '0';
          cmmd(1 downto 0) <= "10";			
          
          dq  <= data;
          
	when search68_b => 

          s_tr_dq   <= '0'; 
          s_tr_cmdv <= '0'; 
          s_tr_cmd  <= '0';	
          
          cmmdv     <= '1';
          cmmd(8 downto 6) <= ssr_reg_index;
          cmmd(5 downto 2) <= comparand_reg_index;
          cmmd(1 downto 0) <= "10";
          
          dq  <= data; --same data as previous;
          
          if (cmd(1 downto 0) = "10" and cmdv = '1') then
            ready_s <= '0';
          else
            ready_s <= '1';
          end if;
	when others =>
          --tristate outputs

          s_tr_dq   <= '1'; 
          s_tr_cmdv <= '1'; 
          s_tr_cmd  <= '1';
          ready_s   <= '1';

          
          cmmd <= (others => 'Z');
          cmmdv     <= 'Z';
          dq  <= (others => 'Z');
          
          
      end case;
      
    end process width68_logic; 

  end generate generate_68_bit_logic;  



  
  generate_136_bit_logic: if (datawidth = "01") generate

    width138_nextstate: process (CurrState_search, cmd, cmdv)
    begin  -- process width138_nextstate
      case CurrState_search is
        when idle_s => 
          if(cmd(1 downto 0)= "10" and cmdv = '1') then
            NextState_search <= search136_a;
          else
            NextState_search <= idle_s;
          end if;
        when search136_a =>
          
          NextState_search <= search136_b;
        when search136_b =>
          if (cmd(1 downto 0) = "10" and  cmdv = '1') then
            NextState_search <= search136_a;
          else 
            NextState_search<=idle_s;
          end if;
          
        when others => 
          NextState_search<=idle_s;
      end case;
    end process width138_nextstate; 
    
    width136: process(cmd, cmdv, comparand_reg_index, ssr_reg_index, CurrState_search, data, gmri, sadr)
    begin
      case CurrState_search is
        when idle_s =>

          s_tr_dq   <= '1'; 
          s_tr_cmdv <= '1'; 
          s_tr_cmd  <= '1';	
          ready_s   <= '1';
          
          cmmd <= (others => 'Z');
          cmmdv     <= 'Z';
          dq  <= (others => 'Z');

          --tristate outputs

      when search136_a =>

      s_tr_dq   <= '0'; 
      s_tr_cmdv <= '0'; 
      s_tr_cmd  <= '0';	
      ready_s   <= '0';
      
      cmmdv     <= '1';
      
      cmmd(8 downto 6) <= sadr; 	--SRAM addr for SRAM support
      cmmd(5 downto 3) <= gmri;
      cmmd(2)   <= '0';
      cmmd(1 downto 0) <= "10"; 
      
      dq <= data;



      when search136_b =>

      s_tr_dq   <= '0'; 
      s_tr_cmdv <= '0'; 
      s_tr_cmd  <= '0';	
      
      cmmdv     <= '1';
      
      cmmd(8 downto 6) <= ssr_reg_index;
      cmmd(5 downto 2) <= comparand_reg_index;
      cmmd(1 downto 0) <= "10";
      
      dq  <= data;
      
      if (cmd(1 downto 0) = "10" and  cmdv = '1') then
        ready_s  <= '0';
      else 
        ready_s  <= '1';
      end if;
      when others =>

      s_tr_dq   <= '1'; 
      s_tr_cmdv <= '1'; 
      s_tr_cmd  <= '1';	
      ready_s   <= '1';
      
      cmmd <= (others => 'Z');
      cmmdv     <= 'Z';
      dq  <= (others => 'Z');

      --tristate outputs
    end case;
    
  end process width136;
end generate generate_136_bit_logic;


generate_272_bit_logic: if (datawidth = "10") generate

  width272_nextstate: process (CurrState_search, cmd, cmdv)
  begin  -- process width272_nextstate
    case CurrState_search is
      when idle_s => 
                     if(cmd(1 downto 0) = "10" and cmdv = '1') then
                       NextState_search <= search272_a;
                     else
                       NextState_search <= idle_s;
                     end if;
      when search272_a =>
        NextState_search <= search272_b;
      when search272_b =>
        NextState_search <= search272_c;
      when search272_c =>
        NextState_search <= search272_d;

      when search272_d =>
        if (cmd(1 downto 0) = "10" and cmdv = '1') then 
          NextState_search <= search272_a;
        else 
          NextState_search <= idle_s;
        end if;

      when others =>
        NextState_search <= idle_s;
    end case;
  end process width272_nextstate;

  width272: process(cmd, cmdv, CurrState_search, data, gmri, sadr)
  begin
    case CurrState_search is
      when idle_s =>
        --tristate outputs

        s_tr_dq   <= '1'; 
        s_tr_cmdv <= '1'; 
        s_tr_cmd  <= '1';	
        ready_s   <= '1';

        cmmd <= (others => 'Z');
        cmmdv     <= 'Z';
        dq <= (others => 'Z');
        
      when search272_a =>

        s_tr_dq   <= '0'; 
        s_tr_cmdv <= '0'; 
        s_tr_cmd  <= '0';	
        ready_s   <= '0';

        cmmdv     <= '1';
        
        cmmd(8 downto 6) <= "000";
        cmmd(5 downto 3) <= gmri;
        cmmd(2)   <= '1';
        cmmd(1 downto 0) <= "10";
        
        dq <= data;

      when search272_b =>

        s_tr_dq   <= '0'; 
        s_tr_cmdv <= '0'; 
        s_tr_cmd  <= '0';
        
        ready_s   <= '0';

        cmmdv     <= '1';
        
        cmmd(8 downto 6) <= "000";
        cmmd(5 downto 3) <= gmri;
        cmmd(2)   <= '1';
        cmmd(1 downto 0) <= "10";
        
        dq <= data; --next data;

      when search272_c =>

        s_tr_dq   <= '0'; 
        s_tr_cmdv <= '0'; 
        s_tr_cmd  <= '0';	
        ready_s   <= '0';

        cmmdv     <= '1';
        
        cmmd(8 downto 6) <= sadr; --SRAM addr for SRAM support
        cmmd(5 downto 3) <= gmri;
        cmmd(2)   <= '0';
        cmmd(1 downto 0) <= "10"; 
        
        dq <= data;--next data;

      when search272_d =>

        s_tr_dq   <= '0'; 
        s_tr_cmdv <= '0'; 
        s_tr_cmd  <= '0';	

        cmmdv     <= '1';
        
        cmmd(8 downto 6) <= sadr; --SRAM addr for SRAM support
        cmmd(5 downto 3) <= gmri;
        cmmd(2)   <= '0';
        cmmd(1 downto 0) <= "10"; 
        
        dq <= data; --next data;
        if (cmd(1 downto 0) = "10" and cmdv = '1') then 
          ready_s <= '0';
        else 
          ready_s <= '1';
        end if;

      when others =>

        s_tr_dq   <= '1'; 
        s_tr_cmdv <= '1'; 
        s_tr_cmd  <= '1';	
        ready_s   <= '1';

        cmmd <= (others => 'Z');
        cmmdv     <= 'Z';
        dq <= (others => 'Z');
        
        
    end case;
  end process width272;

end generate generate_272_bit_logic;

-- Current State Logic (sequential)
                        next_state_logic: process (clk2x, rst)
                        begin  -- process next_state_logic
                          if rst = '1' then                    -- asynchronous reset
                            CurrState_search <= idle_s;
                          elsif clk2x'event and clk2x = '1' then  -- rising clock edge
                            CurrState_search <= NextState_search;
                            
                          end if;
                        end process next_state_logic;
                        
                        end search_arch;




