/******************************************************************************
*
*    File Name:  init.v
*      Version:  1.1
*         Date:  January 24, 2001
*  Description:  Initialization state machine for interface with Lara Networks
*                CAM
* Dependencies:  defines.v
*
*      Company:  Xilinx
*
*
*   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
*                WHATSOEVER.
*
*                Copyright (c) 2001 Xilinx, Inc.
*                All rights reserved
*
*   History:     Ver 1.1 - Resolved all latch creating issues.
*                          Re-designed state machines to separate
*                          next state logic from state outputs. 
*                          Added addition comments for clarification.  
*
******************************************************************************/
`include "defines.v"
`timescale 10 ps/10 ps
                    
module init (   clk, 
                rst,
                start, 
                cmmd,            
                cmmdv, 
                dq, 
                i_tr_dq, 
                i_tr_cmdv, 
                i_tr_cmd, 
                ready_i,
                id, 
                id_enable,
                last);

input clk, rst;

// start = 1 indicates that the initialize state
// machine can start execution
input start;          
input [4:0]id;

//last = 1 indicates last CAM to initialize
input last;
         
output [8:0]cmmd;  //command
output [67:0]dq;   //data/address bus
output cmmdv;      //command valid

//ready_i indicates that the initialize state machine can start execution               
output ready_i;   
output i_tr_dq;    // tri_state enable signals
output i_tr_cmdv;  // tristate control for command valid signal
output i_tr_cmd;   

//id_enable = 1 indicates that the user can change the value of id
output id_enable;

reg cmmdv, ready_i, id_enable;
reg [67:0]dq;
reg [8:0]cmmd;
reg i_tr_dq, i_tr_cmdv, i_tr_cmd;

//Define width parameter
`ifdef width68
  parameter width = 2'b00;
`endif
`ifdef width136
  parameter width = 2'b01;
`endif
`ifdef width272
  parameter width = 2'b10;
`endif

//Define depth parameter
`ifdef depth1
  parameter tlsz = 2'b00;
`endif 
`ifdef depth2_8
  parameter tlsz = 2'b01;
`endif
`ifdef depth9_31
  parameter tlsz = 2'b10;
`endif

//state assignment
parameter idle        = 3'b000, 
          init2_cm  = 3'b001, 
          init2_gmr = 3'b010, 
          init3_cm  = 3'b011, 
          init3_gmr = 3'b100, 
          init_cm   = 3'b101, 
          init_gmr  = 3'b110;

// state machine signals
reg [2:0]CurrState_initialize, NextState_initialize;


// a process which determines what the outputs should be for a given state
always @ (start or cmmdv or dq or CurrState_initialize or id or last)
begin
  case (CurrState_initialize)
    idle:
      begin
        //tristate all outputs:
        i_tr_dq   = 1'b1; 
        i_tr_cmdv = 1'b1; 
        i_tr_cmd  = 1'b1;               
        ready_i   = 1'b1;
        id_enable = 1'b1;
                                               
        cmmd[8:0] = 9'bzzzzzzzzz;
        cmmdv     = 1'bz;
        dq[67:0]  = 68'hzzzzzzzzzzzzzzzzz;                                                           
      end
    init_cm:
      begin
        i_tr_dq   = 1'b0; 
        i_tr_cmdv = 1'b0; 
        i_tr_cmd  = 1'b0;
        ready_i   = 1'b0;
        id_enable = 1'b0;
                                                            
        //initialize command register
        //consult Lara Networks CAM datasheet
        //  for details on register meanings

        // note that cmmd[2:0] indicates a single write,
        //  cmmd[8:3] are for parameters
        cmmd  = 9'b000000001; 
        cmmdv = 1'b1;

        dq[67:26] = 41'h00000000000;  // reserved command reg area
        dq[25:21] = id;  // give CAM ID
        dq[20:19] = 2'b11;  // indicate internal register write
        dq[18:6]  = 13'h0000;   // reserved command reg area
        dq[5:0]   = 6'b111000;  // register address
      end
    init2_cm:
      begin
        i_tr_dq   = 1'b0; 
        i_tr_cmdv = 1'b0; 
        i_tr_cmd  = 1'b0;
        ready_i   = 1'b0;
        id_enable = 1'b0;
                                                            
        //initialize the command register
        // note that cmmd[2:0] indicates a single write,
        //  cmmd[8:3] are for parameters
        // preserve cmmd so we don't get latches, notice
        // that the command indicator is set to invalid
        cmmd  = 9'b000000001; 
        cmmdv = 1'b0;
 
        dq[67:17] = 51'h0000000000000;  // reserved command reg area
 
        //table size configuration for each of the 4 quadrants
        dq[16:9]  = {width, width, width, width}; 
                                                            
        if (last == 1'b1)
          dq[8:7] = 2'b11; //Last CAM
        else
          dq[8:7] = 2'b00;
                                                            
        dq[6:4] = 3'b000; //0 latency for SSF, SSV, and ACK
        dq[3:2] = tlsz;   // table size        
        dq[1]   = 1'b1;           //device enable
        dq[0]   = 1'b0;           //software reset if 1;
      end
    init3_cm:
      begin
        i_tr_dq   = 1'b0; 
        i_tr_cmdv = 1'b0; 
        i_tr_cmd  = 1'b0;
        ready_i   = 1'b0;
        id_enable = 1'b0;
                                                            
        //initialize the command register
        // allow one clock cycle to finish command reg write.
        cmmd  = 9'b000000001; 
        cmmdv = 1'b0;
        
        dq[67:0] = 68'hz;
      end
      init_gmr:
        begin
          i_tr_dq   = 1'b0; 
          i_tr_cmdv = 1'b0; 
          i_tr_cmd  = 1'b0;
          ready_i   = 1'b0;
          id_enable = 1'b0;
                                                            
          //initialize Global Mask Reg
          // note that cmmd[2:0] indicates a single write,
          //  cmmd[8:3] are for parameters
          cmmd  = 9'b000000001;
          cmmdv = 1'b1;
         
          dq[67:26] = 41'h0000000000; // reserved
          dq[25:21] = id;  //CAM ID
          dq[20:19] = 2'b11;  // internal reg write
          dq[18:6]  = 13'h0000; // reserved
          dq[5:0]   = 6'b100000;  // first global mask register
        end
      init2_gmr:
        begin
          i_tr_dq   = 1'b0; 
          i_tr_cmdv = 1'b0; 
          i_tr_cmd  = 1'b0;
          ready_i   = 1'b0;
          id_enable = 1'b1;
                                                            
          //initialize Global Mask Reg
          // note that cmmd[2:0] indicates a single write,
          //  cmmd[8:3] are for parameters
          // preserve cmmd so we don't get latches, notice
          // that the command indicator is set to invalid
          cmmd = 9'b000000001;
          cmmdv     = 1'b0;
          
          dq[67:0]  = 68'hFFFFFFFFFFFFFFFFF;  // give global mask pattern
    end       
      init3_gmr:        
        begin  
          i_tr_dq   = 1'b0; 
          i_tr_cmdv = 1'b0; 
          i_tr_cmd  = 1'b0;
          ready_i   = 1'b1;
        id_enable = 1'b1;
  
          //initialize Global Mask Reg
          // note that cmmd[2:0] indicates a single write,
          //  cmmd[8:3] are for parameters
          // preserve cmmd so we don't get latches, notice
          // that the command indicator is set to invalid
          cmmd = 9'b000000001;
      cmmdv     = 1'b0;
  
          dq[67:0]  = 68'hz; // allow one clock cycle to finish reg write
    end
      default:
      begin
          // default state so we don't get latches 
          //tristate all outputs:
          i_tr_dq   = 1'b1; 
          i_tr_cmdv = 1'b1; 
      i_tr_cmd  = 1'b1;               
        ready_i   = 1'b1;
      id_enable = 1'b1;

        cmmd[8:0] = 9'bzzzzzzzzz;
          cmmdv     = 1'bz;
  
          dq[67:0]  = 68'hzzzzzzzzzzzzzzzzz;
        end
  endcase
end

// Create a process that describes the next state logic
always @(start or CurrState_initialize)
begin
  case (CurrState_initialize)
    idle:
    begin
      if(start == 1'b1)
      NextState_initialize = init_cm;
    else
      NextState_initialize = idle;
      end
    init_cm:
      begin
        NextState_initialize = init2_cm;
      end
    init2_cm:
      begin
        NextState_initialize = init3_cm;
      end
    init3_cm:
      begin
        NextState_initialize = init_gmr;
      end
    init_gmr:
      begin
        NextState_initialize = init2_gmr;
      end
    init2_gmr:
      begin
        NextState_initialize = init3_gmr;
      end
    init3_gmr:
    begin
      if (start == 1'b1 )
        NextState_initialize = init_cm;
      else
        NextState_initialize = idle;
      end
    default:
      NextState_initialize = idle;
  endcase
end


// Current State Logic
always @ (posedge clk or posedge rst)
begin
  if (rst)
    CurrState_initialize <= idle;
  else
    CurrState_initialize <= NextState_initialize;
end

endmodule



















