/******************************************************************************
*
*    File Name:  rw.v
*      Version:  1.0
*         Date:  January 24, 2001
*  Description:  Read/Write state machine for interface with Lara Networks CAM
* Dependencies:  defines.v
*
*      Company:  Xilinx
*
*
*   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
*                WHATSOEVER.
*
*                Copyright (c) 2000 Xilinx, Inc.
*                All rights reserved
*
*   History:     Ver 1.1 - Resolved all latch creating issues.
*                          Re-designed state machines to separate
*                          next state logic from state outputs. 
*                          Fixed burst writes to Mask registers.
*                          Fixed burst write addressing size problem. 
*                          Added addition comments for clarification.  
*
******************************************************************************/
module rw (	clk, 
            rst, 
            user_command,
            cmmd, 
            cmmdv, 
            data, 
            data_back, 
            data_rw, 
            dq,  
            eot_in, 
            ack,  
            rw_tr_dq, 
            rw_tr_cmdv, 
            rw_tr_cmd, 
            ready_rw, 
            gmri, 
            id);
     
input clk, rst;
input eot_in;
input ack;     
input [5:0]user_command;
input [4:0]id;
input [2:0]gmri;
input [67:0]data;//output of register rw_in
input [67:0]data_back;

output cmmdv, ready_rw;
output [8:0]cmmd;
output [67:0]data_rw,dq;
output rw_tr_dq, rw_tr_cmdv, rw_tr_cmd; //tri-state control signals

reg [8:0]cmmd;
reg cmmdv;
reg ready_rw;
reg [67:0]dq, data_rw;
reg [8:0]burst_cnt_syn;
reg [8:0]burst_cnt_com;
reg rw_tr_dq, rw_tr_cmdv, rw_tr_cmd;

wire [1:0]dmr;
wire burst;
wire [1:0]cmd;
wire cmdv;
reg [8:0] burst_length; //data[23:15] = burst_length

//internal nodes
//reg [67:0] data_rw_int;


// user_command
//    [5]        [4:3]      [2:1]       [0]
//  burst         dmr        cmd        cmdv

assign burst = user_command[5];
assign dmr = user_command[4:3];
assign cmd[1:0]=user_command[2:1];
assign cmdv=user_command[0];

//state encoding 
parameter idle_a = 4'b0000, 
          read1  = 4'b0001, 
          read2  = 4'b0010, 
          read3  = 4'b0011, 
          write1 = 4'b0100, 
          write2 = 4'b0101, 
          write3 = 4'b0110, 
          write4 = 4'b0111, 
          write5 = 4'b1000, 
          write6 = 4'b1001,
       write_nop = 4'b1010;

reg [3:0]CurrState_rw, NextState_rw;


//NextState logic
always @(CurrState_rw  or cmd or burst or cmdv or  eot_in or ack or burst_cnt_syn or burst_length)
begin
  case (CurrState_rw)
    idle_a:
      begin
        if (cmd[1:0]==2'b01 && cmdv==1'b1 || (cmd[1:0]==2'b00 && burst==1'b1 && cmdv==1'b1) )
          begin
            //Enter this state if you are doing a burst
            //  read (must program burst read reg first), or
            //if you are doing a write (burst or not)
            NextState_rw = write1;
          end
        else if (cmd[1:0] == 2'b00 && burst == 1'b0 && cmdv==1'b1)
          begin
            NextState_rw = read1; 
            //Enter this state if you are doing a non-burst read.  
          end
        else
          NextState_rw = idle_a;
      end  
    read1:
      begin
        if(burst == 1'b1)
          NextState_rw = read2;
        else
          NextState_rw = read3;  
      end
    read2:
      begin
        if (eot_in==1'b1)
          NextState_rw = idle_a;
        else
          NextState_rw = read2;
      end
    read3:
      begin
        if(ack ==1'b1) 
          NextState_rw = idle_a;
        else 
          NextState_rw = read3;      
      end
    write1:
      begin
        NextState_rw = write2;
      end
    write2:
      begin
        if (cmd[1:0]==2'b01 && burst==1'b1)    
          NextState_rw = write_nop; //burst write
        else if (cmd[1:0]==2'b00 && burst==1'b1) 
          NextState_rw = write_nop; //burst read
        else   
          NextState_rw = idle_a; //finished single write
      end
    write_nop:
      begin
        if (cmd[1:0]==2'b01) 
          NextState_rw = write3;
        else
          NextState_rw = read1;
      end
    write3:
      begin
        NextState_rw = write4;
      end
    write4:
      begin
        NextState_rw = write5;
      end
    write5:
      begin
        if (burst_cnt_syn == burst_length) 
          NextState_rw = write6;
        else   
          NextState_rw = write5;
      end
    write6:
      begin
        NextState_rw = idle_a;
      end
    default:
       NextState_rw = idle_a;
  endcase 
end

// state output logic
always @ (cmd or cmdv or burst or data or ack or dmr or data_back or data_rw or eot_in 
        or CurrState_rw or id or gmri or burst_cnt_syn)
begin
  case (CurrState_rw)
    idle_a:
      begin
        //tri-state all outputs
        rw_tr_dq   = 1'b1;   
        rw_tr_cmd   = 1'b1;   
        rw_tr_cmdv   = 1'b1; 
        ready_rw  = 1'b1;
      
        cmmd[8:0]   = 9'bzzzzzzzzz;
        cmmdv     = 1'bz;
        dq[67:0]   = 68'hzzzzzzzzzzzzzzzzz;
      end
    read1:
      begin
        rw_tr_dq   = 1'b0;   
        rw_tr_cmd  = 1'b0;   
        rw_tr_cmdv = 1'b0;  
        ready_rw   = 1'b0;
        
        cmmd[1:0] = 2'b00;
        cmmd[2]   = burst;
        cmmd[8:3] = 6'b000000;
        cmmdv     = 1'b1;
      
        dq[67:30] = 37'h0000000000;
        dq[29]    = 1'b0;
        dq[28:26] = 3'b000;
        dq[25:21] = id;
        dq[20:19] = dmr;//00:data read, 01:mask read, 10:SRAM read, 11:register read
        dq[18:15] = 4'h0;
        dq[14:0]  = data[14:0]; //addr: for burst read, these bits are don't cares
      end
    read2:
      begin
        //burst read only
        rw_tr_dq   = 1'b1;   
        rw_tr_cmd  = 1'b1;   
        rw_tr_cmdv = 1'b0; 
        ready_rw   = 1'b0;
  
        cmmd[1:0] = 2'b00;
        cmmd[2]   = burst;
        cmmd[8:3] = 6'b000000;
        cmmdv      = 1'b0;
        // same ase state read1 since there is only one way to get here
        dq[67:30] = 37'h0000000000;
        dq[29]    = 1'b0;
        dq[28:26] = 3'b000;
        dq[25:21] = id;
        dq[20:19] = dmr;//00:data read, 01:mask read, 10:SRAM read, 11:register read
        dq[18:15] = 4'h0;
        dq[14:0]  = data[14:0]; //addr: for burst read, these             
      end
    read3:
      begin
        //single read or last cycle of burst read  
        rw_tr_dq   = 1'b1;   
        rw_tr_cmd  = 1'b0;   
        rw_tr_cmdv = 1'b0; 
        ready_rw   = 1'b1;

        cmmd[1:0] = 2'b00;
        cmmd[2]   = burst;
        cmmd[8:3] = 6'b000000;
        cmmdv      = 1'b0;

        // same ase state read1 since there is only one way to get here
        dq[67:30] = 37'h0000000000;
        dq[29]    = 1'b0;
        dq[28:26] = 3'b000;
        dq[25:21] = id;
        dq[20:19] = dmr;//00:data read, 01:mask read, 10:SRAM read, 11:register read
        dq[18:15] = 4'h0;
        dq[14:0]  = data[14:0]; //addr: for burst read, these       
      end
    write1:
      begin
        rw_tr_dq   = 1'b0;   
        rw_tr_cmd  = 1'b0;   
        rw_tr_cmdv = 1'b0; 
        ready_rw   = 1'b0;
      
        //write to burst read register
        if(cmd[1:0]==2'b00 && burst==1'b1)
          begin
            cmmdv     = 1'b1;
            cmmd[1:0] = 2'b01;
            cmmd[2]   = 1'b0;
            cmmd[8:3] = 6'b000000;
      
            dq[5:0]   = 6'b111010; //dec address 58 for burst read reg
            dq[18:6]  = 13'h0000;
            dq[20:19] = 2'b11; //for register write
            dq[25:21] = id;
            dq[28:26] = 3'b000;
            dq[29]    = 1'b0;
            dq[67:30] = 37'h0000000000;
          end
        //write to burst write register
        else if(cmd[1:0]==2'b01 && burst==1'b1)
          begin
            cmmdv     = 1'b1;
            cmmd[2]   = 1'b0;
            cmmd[1:0] = 2'b01;
            cmmd[5:3] = gmri; //GMR index for write;
            cmmd[8:6] = 3'b000;
      
            dq[5:0]   = 6'b111011; //burst write reg address 59;
            dq[18:6]  = 13'h0000;
            dq[20:19] = 2'b11;
            dq[25:21] = id;
            dq[28:26] = 3'b000;
            dq[29]    = 1'b0;
            dq[67:30] = 37'h0000000000;
          end
        else
         // if we're here, this can be nothing other than a single write
          begin
            cmmdv     = 1'b1;
            cmmd[2]   = 1'b0;
            cmmd[1:0] = 2'b01;
            cmmd[5:3] = gmri;//GMR index for write;
            cmmd[8:6] = 3'b000;
      
            dq[14:0]  = data[14:0]; //addr
            dq[18:15] = 4'h0;
            dq[20:19] = dmr; //00:data write, 01:mask write, 10: SRAM write, 11:register write
            dq[25:21] = id;
            dq[28:26] = 3'b000;
            dq[29]    = 1'b0;
            dq[67:30] = 37'h0000000000;
          end // if (cmd[1:0]==2'b01 && burst==1'b0)
        end
      write2:
        begin
          rw_tr_dq   = 1'b0;   
          rw_tr_cmd  = 1'b0;   
          rw_tr_cmdv = 1'b0;  
          ready_rw  = 1'b0;
      
          //write to burst read register or burst write register
          //dq[14:0]   = data[14:0] = addr: 15-bit starting address for burst
          //dq[27:19]  = data[23:15] = burst_length
          // rest is don't care

          cmmdv      = 1'b0;  
      
          if (cmd[1:0]==2'b01 && burst==1'b1)    // burst write
            begin
              cmmd[2]   = 1'b0;
              cmmd[1:0] = 2'b01;
              cmmd[5:3] = gmri; //GMR index for write;
              cmmd[8:6] = 3'b000;
	          ready_rw=1'b0;

              dq[18:15]  = 4'h0;
              dq[67:28] = 39'h0000000000;
  
              dq[14:0]   = data[14:0];
              dq[27:19]  = data[23:15];
            end
          else if (cmd[1:0]==2'b00 && burst==1'b1) // burst read
            begin
              cmmd[1:0] = 2'b01;
              cmmd[2]   = 1'b0;
              cmmd[8:3] = 6'b000000;
              ready_rw=1'b0;

              dq[18:15]  = 4'h0;
              dq[67:28] = 39'h0000000000;
              dq[14:0]   = data[14:0];
              dq[27:19]  = data[23:15];
            end
          else   
            begin
              // finished signal write
              cmmd[2]   = 1'b0;
              cmmd[1:0] = 2'b01;
              cmmd[5:3] = gmri;//GMR index for write;
              cmmd[8:6] = 3'b000;
              ready_rw=1'b1;
              dq[67:0]   = data[67:0];
            end
        end
      write_nop:
        begin
          rw_tr_dq   = 1'b1;   
          rw_tr_cmd  = 1'b1;   
          rw_tr_cmdv = 1'b1;  
          ready_rw  = 1'b0;
          cmmdv      = 1'b0;  
        
          if (cmd[1:0]==2'b01 && burst==1'b1)    
            begin
              cmmd[2]   = 1'b0;
              cmmd[1:0] = 2'b01;
              cmmd[5:3] = gmri; //GMR index for write;
              cmmd[8:6] = 3'b000;
	          ready_rw=1'b0;

              dq[18:15]  = 4'h0;
              dq[67:28] = 39'h0000000000;
              dq[14:0]   = data[14:0];
              dq[27:19]  = data[23:15];
            end
          else if (cmd[1:0]==2'b00 && burst==1'b1) 
            begin
              cmmd[1:0] = 2'b01;
              cmmd[2]   = 1'b0;
              cmmd[8:3] = 6'b000000;
              ready_rw=1'b0;

              dq[18:15]  = 4'h0;
              dq[67:28] = 39'h0000000000;
              dq[14:0]   = data[14:0];
              dq[27:19]  = data[23:15];
            end
          else
            begin
              cmmd[2]   = 1'b0;
              cmmd[1:0] = 2'b01;
              cmmd[5:3] = gmri;//GMR index for write;
              cmmd[8:6] = 3'b000;
              ready_rw=1'b1;
              dq[67:0]   = data[67:0];
            end
        end  
      write3:   // h6
        begin
          rw_tr_dq   = 1'b0;   
          rw_tr_cmd  = 1'b0;   
          rw_tr_cmdv = 1'b0;  
          ready_rw  = 1'b0;
      
          //supply address
          cmmdv     = 1'b1;
          cmmd[1:0] = 2'b01;
          cmmd[2]   = burst;
          cmmd[8:3] = 6'b000000;
      
          dq[14:0]  = data[14:0]; //address
          dq[18:15] = 5'b00000; // reserved
          dq[20:19] = dmr;   // data, mask or reg
          dq[25:21] = id;  // CAM ID
          dq[67: 26] = 42'h00000000000; // reserved
        end
      write4: //          write4 = 4'b0111 = h7 
        begin
          rw_tr_dq   = 1'b0;   
          rw_tr_cmd  = 1'b0;   
          rw_tr_cmdv = 1'b0;  
          ready_rw   = 1'b0;

          cmmdv      = 1'b0;
          cmmd[1:0] = 2'b01;
          cmmd[2]   = burst;
          cmmd[8:3] = 6'b000000;
      
          //supply first data
          dq[67:0]   = data;
        end
      write5: //          write5 = 4'b1000 = h8 
        begin
          rw_tr_dq   = 1'b0;   
          rw_tr_cmd  = 1'b0;   
          rw_tr_cmdv = 1'b0; 
          ready_rw   = 1'b0;

          cmmdv      = 1'b0;
          cmmd[1:0] = 2'b01;
          cmmd[2]   = burst;
          cmmd[8:3] = 6'b000000;
      
          //suppy data
          dq[67:0]   = data[67:0];
    	end
      write6: //          write6 = 4'b1001 = h9
        begin
          rw_tr_dq   = 1'b1;   
          rw_tr_cmd  = 1'b0;   
          rw_tr_cmdv = 1'b0;  
          ready_rw   = 1'b1;

          cmmdv      = 1'b0;
          cmmd[1:0] = 2'b01;
          cmmd[2]   = burst;
          cmmd[8:3] = 6'b000000;
          dq[67:0]   = data[67:0];
        end
      default:
        begin      //tri-state all outputs
          rw_tr_dq   = 1'b1;   
          rw_tr_cmd   = 1'b1;   
          rw_tr_cmdv   = 1'b1; 
          ready_rw  = 1'b1;
      
          cmmd[8:0]   = 9'bzzzzzzzzz;
          cmmdv     = 1'bz;
          dq[67:0]   = 68'hzzzzzzzzzzzzzzzzz;
        end  
  endcase
end

//assign data_rw = data_rw_int;
// registered outputs
always @(posedge clk or posedge rst)
begin
  if(rst == 1'b1)
    begin
      data_rw <= 68'h00000000000000000;
      burst_length = 7'b0000000;
    end
  else
    begin
      if (((CurrState_rw == read2) || (CurrState_rw == read3)) && (ack == 1'b1))
        data_rw <= data_back;
      if (CurrState_rw == write2)
        burst_length = data[23:15];
    end
end


// burst counter
always @(posedge clk or posedge rst)
begin  // process burst_count
  if (rst == 1'b1)              
    begin 
      burst_cnt_com = 9'b000000010;
      burst_cnt_syn = 9'b000000000;
    end
  else    // rising clock edge
    begin
      burst_cnt_syn = burst_cnt_com;

      if (CurrState_rw == write4)
        burst_cnt_com = 9'b000000010;
      else if (CurrState_rw == write5)
        burst_cnt_com = burst_cnt_com + 9'b000000001;
    end  
end


// Current State Logic (sequential)
always @ (posedge clk or posedge rst)
begin
  if(rst) 
    begin
      CurrState_rw = idle_a;
    end
  else   
    begin
      CurrState_rw = NextState_rw;
    end
end

endmodule




















