/******************************************************************************
*
*    File Name:  search.v
*      Version:  1.1
*         Date:  January 24, 2001
*  Description:  Search state machine for interface with Lara Networks CAM
* Dependencies:  defines.v
*
*      Company:  Xilinx
*
*
*   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
*                WHATSOEVER.
*
*                Copyright (c) 2001 Xilinx, Inc.
*                All rights reserved
*   
*   History:     Ver 1.1 - Resolved all latch creating issues.
*                          Re-designed state machines to separate
*                          next state logic from state outputs. 
*
******************************************************************************/
`timescale 1ps / 1ps
`include "defines.v"

module search (  clk2x, 
                 rst,
                 user_command, 
                 cmmd, 
                 cmmdv,
                 dq, 
                 data,  
                 s_tr_dq, 
                 s_tr_cmdv, 
                 s_tr_cmd, 
                 ready_s, 
                 gmri,
				 sadr,
                 ssr_reg_index,
                 comparand_reg_index
              );


input clk2x, rst;
input [2:0]gmri;
input [67:0]data;
input [5:0]user_command;
input [2:0]sadr;
input [2:0]ssr_reg_index;
input [3:0]comparand_reg_index;

output cmmdv;
output [8:0]cmmd;
output [67:0]dq;
output s_tr_dq, s_tr_cmdv, s_tr_cmd, ready_s; //tri_state enables

reg [67:0]dq;
reg [8:0]cmmd;
reg cmmdv, ready_s;
reg s_tr_dq, s_tr_cmdv, s_tr_cmd;

wire [1:0]cmd;
wire cmdv;


// user_command
//    [5]        [4:3]      [2:1]       [0]
//  burst         dmr        cmd        cmdv


assign cmd[1:0]  = user_command[2:1];
assign cmdv  = user_command[0];

// BINARY ENCODED state machine: search
parameter idle_s      = 4'b0000,
    search136_a = 4'b0001, 
          search136_b = 4'b0010, 
          search272_a = 4'b0011, 
          search272_b = 4'b0100, 
          search272_c = 4'b0101, 
          search272_d = 4'b0110, 
          search68_a  = 4'b0111, 
          search68_b  = 4'b1000;

reg [3:0]CurrState_search, NextState_search;


// since this can be configured to be different widths, create
// different state machines depending on this value
`ifdef width68
//create the next state logic
always @ (cmd or cmdv or comparand_reg_index or ssr_reg_index or CurrState_search or data or gmri or sadr)
begin
  case (CurrState_search)
    idle_s:
    begin
      if(cmd[1:0]==2'b10 && cmdv==1'b1) NextState_search=search68_a;
      else NextState_search=idle_s;
    end
  
    search68_a:
    begin
      NextState_search = search68_b;
    end

    search68_b:
    begin
      if (cmd[1:0]==2'b10 && cmdv==1'b1) 
      begin
        NextState_search = search68_a;
      end
      else
      begin
        NextState_search=idle_s;
      end
    end
    default: NextState_search=idle_s;
  endcase
end 

//create a block for the state outputs
always @ (cmd or cmdv or  comparand_reg_index or ssr_reg_index or CurrState_search or data or gmri or sadr)
begin
  case (CurrState_search)
    idle_s:
    begin
      //tristate outputs

      s_tr_dq   = 1'b1; 
      s_tr_cmdv = 1'b1; 
      s_tr_cmd  = 1'b1;
      ready_s   = 1'b1;

          
      cmmd[8:0] = 9'bzzzzzzzzz;
      cmmdv     = 1'bz;
      dq[67:0]  = 68'hzzzzzzzzzzzzzzzzz;    
      
                end  
    search68_a:
    begin

      s_tr_dq   = 1'b0; 
      s_tr_cmdv = 1'b0; 
      s_tr_cmd  = 1'b0;  
      ready_s   = 1'b0;
      
      cmmdv     = 1'b1;
      
      cmmd[8:6] = sadr; //SRAM addr for SRAM support
      cmmd[5:3] = gmri; 
      cmmd[2]    = 1'b0;
      cmmd[1:0] = 2'b10;      
      
      dq[67:0]  = data[67:0];
      
  end
    search68_b:
    begin

      s_tr_dq   = 1'b0; 
      s_tr_cmdv = 1'b0; 
      s_tr_cmd  = 1'b0;  
   
      cmmdv     = 1'b1;
      cmmd[8:6] = ssr_reg_index;
      cmmd[5:2] = comparand_reg_index;
      cmmd[2]    = 1'b0;
      cmmd[1:0] = 2'b10;
      
      dq[67:0]  = data[67:0]; //same data as previous;
      
      if (cmd[1:0]==2'b10 && cmdv==1'b1) 
      begin
        ready_s=1'b0;
      end
      else
      begin
        ready_s=1'b1;
      end
    end
    default:
          begin
      s_tr_dq   = 1'b1; 
      s_tr_cmdv = 1'b1; 
      s_tr_cmd  = 1'b1;
      ready_s   = 1'b1;

          
      cmmd[8:0] = 9'bzzzzzzzzz;
      cmmdv     = 1'bz;
      dq[67:0]  = 68'hzzzzzzzzzzzzzzzzz;    
      end
  endcase
end 
`endif 

`ifdef width136
// make the next state logic
always @ (cmd or cmdv or comparand_reg_index or ssr_reg_index or 
       CurrState_search or data or gmri or sadr)
begin
  case (CurrState_search)
    idle_s:
    begin

      if(cmd[1:0]==2'b10 && cmdv==1'b1) NextState_search=search136_a;
      else NextState_search=idle_s;
    end

    search136_a:
    begin


      NextState_search = search136_b;

    end
    search136_b:
    begin
      
      if (cmd[1:0]==2'b10 && cmdv==1'b1) 
      begin
        NextState_search = search136_a;
      end      
      else 
      begin
        NextState_search=idle_s;
      end
    end
    default: NextState_search=idle_s;
  endcase
end
// create the outputs for each state
always @ (cmd or cmdv or comparand_reg_index or ssr_reg_index or 
       CurrState_search or data or gmri or sadr)
begin
  case (CurrState_search)
    idle_s:
    begin

      s_tr_dq   = 1'b1; 
      s_tr_cmdv = 1'b1; 
      s_tr_cmd  = 1'b1;  
      ready_s   = 1'b1;
      
      cmmd[8:0] = 9'bzzzzzzzzz;
      cmmdv     = 1'bz;
      dq[67:0]  = 68'hzzzzzzzzzzzzzzzzz;

      //tristate outputs

    end

    search136_a:
    begin

      s_tr_dq   = 1'b0; 
      s_tr_cmdv = 1'b0; 
      s_tr_cmd  = 1'b0;  
      ready_s   = 1'b0;
      
      cmmdv     = 1'b1;
      
      cmmd[8:6] = sadr;   //SRAM addr for SRAM support
      cmmd[5:3] = gmri;
      cmmd[2]   = 1'b0;
      cmmd[1:0] = 2'b10; 
      
      dq[67:0]  = data[67:0];

  
    end
    search136_b:
    begin

      s_tr_dq   = 1'b0; 
      s_tr_cmdv = 1'b0; 
      s_tr_cmd  = 1'b0;  
      
      cmmdv     = 1'b1;
      
      cmmd[8:6] = ssr_reg_index;
      cmmd[5:2] = comparand_reg_index;
      cmmd[1:0] = 2'b10;
      
      dq[67:0]  = data[67:0];
      
      if (cmd[1:0]==2'b10 && cmdv==1'b1) 
      begin
        ready_s  = 1'b0;
      end      
      else 
      begin
          ready_s  = 1'b1;
      end
    end
    default:
          begin
      s_tr_dq   = 1'b1; 
      s_tr_cmdv = 1'b1; 
      s_tr_cmd  = 1'b1;
      ready_s   = 1'b1;

          
      cmmd[8:0] = 9'bzzzzzzzzz;
      cmmdv     = 1'bz;
      dq[67:0]  = 68'hzzzzzzzzzzzzzzzzz;    
      end
  endcase
end

`endif

`ifdef width272
// NextState logic (combinatorial)
always @ (cmd or cmdv or  comparand_reg_index or ssr_reg_index or 
       CurrState_search or data or gmri or sadr)
begin
  case (CurrState_search)
    idle_s:
    begin
      
      if(cmd[1:0]==2'b10 && cmdv==1'b1) NextState_search=search272_a;
      else NextState_search=idle_s;
    end
    search272_a:
    begin


      NextState_search = search272_b;

    end
    search272_b:
    begin

      NextState_search = search272_c;
  
    end
    search272_c:
    begin


      NextState_search = search272_d;

    end
    search272_d:
    begin

    if (cmd[1:0]==2'b10 && cmdv==1'b1) 
      begin
        NextState_search = search272_a;
      end
      else 
      begin
        NextState_search=idle_s;
      end
    end

    default:
      NextState_search=idle_s;
  
  endcase
end
//create the outputs for each state
always @ (cmd or cmdv or  comparand_reg_index or ssr_reg_index or 
       CurrState_search or data or gmri or sadr)
begin
  case (CurrState_search)
    idle_s:
    begin
      //tristate outputs

      s_tr_dq   = 1'b1; 
      s_tr_cmdv = 1'b1; 
      s_tr_cmd  = 1'b1;  
      ready_s   = 1'b1;

      cmmd[8:0] = 9'bzzzzzzzzz;
      cmmdv     = 1'bz;
      dq[67:0]  = 68'hzzzzzzzzzzzzzzzzz;
      
  end
    search272_a:
    begin

      s_tr_dq   = 1'b0; 
      s_tr_cmdv = 1'b0; 
      s_tr_cmd  = 1'b0;  
      ready_s   = 1'b0;

      cmmdv     = 1'b1;
      
      cmmd[8:6] = 3'b000;
      cmmd[5:3] = gmri;
      cmmd[2]   = 1'b1;
      cmmd[1:0] = 2'b10;
       
      dq[67:0]  = data[67:0];

    end
    search272_b:
    begin

      s_tr_dq   = 1'b0; 
      s_tr_cmdv = 1'b0; 
      s_tr_cmd  = 1'b0;
        
      ready_s   = 1'b0;

      cmmdv     = 1'b1;
      
      cmmd[8:6] = 3'b000;
      cmmd[5:3] = gmri;
      cmmd[2]   = 1'b1;
      cmmd[1:0] = 2'b10;
       
      dq[67:0]  = data[67:0]; //next data;

    end
    search272_c:
    begin

      s_tr_dq   = 1'b0; 
      s_tr_cmdv = 1'b0; 
      s_tr_cmd  = 1'b0;  
      ready_s   = 1'b0;

      cmmdv     = 1'b1;
      
      cmmd[8:6] = sadr; //SRAM addr for SRAM support
      cmmd[5:3] = gmri;
      cmmd[2]   = 1'b0;
      cmmd[1:0] = 2'b10; 
      
      dq[67:0]  = data[67:0];//next data;


    end
    search272_d:
    begin

      s_tr_dq   = 1'b0; 
      s_tr_cmdv = 1'b0; 
      s_tr_cmd  = 1'b0;  

      cmmdv     = 1'b1;
      
      cmmd[8:6] = sadr; //SRAM addr for SRAM support
      cmmd[5:3] = gmri;
      cmmd[2]   = 1'b0;
      cmmd[1:0] = 2'b10; 
      
      dq[67:0]  = data[67:0]; //next data;
      if (cmd[1:0]==2'b10 && cmdv==1'b1) 
      begin
        ready_s=1'b0;
      end
      else 
      begin
        ready_s=1'b1;
      end
    end

    default:
    begin
      s_tr_dq   = 1'b1; 
      s_tr_cmdv = 1'b1; 
      s_tr_cmd  = 1'b1;
      ready_s   = 1'b1;

          
      cmmd[8:0] = 9'bzzzzzzzzz;
      cmmdv     = 1'bz;
      dq[67:0]  = 68'hzzzzzzzzzzzzzzzzz;    
    end
  endcase
end
`endif


// Current State Logic (sequential)
always @ (posedge clk2x or posedge rst)
begin
  if (rst)
    CurrState_search <= idle_s;
  else
    CurrState_search <= NextState_search;
end

endmodule















